@@ -53,10 +53,21 @@ class TestDatastore(unittest.TestCase):
 
     checkLength(numelems)
 
-    def test_query(query, expected):
+    k = pkey
+    n = int(numelems)
+    allitems = list(range(0, n))
+
+    def test_query(query, slice):
       for sn in stores:
         try:
+          contents = list(sn.query(Query(pkey)))
+          expected = contents[slice]
           result = list(sn.query(query))
+
+          # make sure everything is there.
+          self.assertTrue(len(contents) == len(allitems))
+          self.assertTrue(all([val in contents for val in allitems]))
+
           self.assertTrue(len(result) == len(expected))
           self.assertTrue(all([val in result for val in expected]))
           #TODO: should order be preserved?
@@ -65,21 +76,11 @@ class TestDatastore(unittest.TestCase):
         except NotImplementedError:
           print 'WARNING: %s does not implement query.' % sn
 
-    for sn in stores:
-      try:
-        dsresults = list(stores[0].query(Query(pkey)))
-      except NotImplementedError:
-        dsresults = []
-
-
-    k = pkey
-    n = int(numelems)
-    test_query(Query(k), list(range(0, n))) # make sure everything is there.
-    test_query(Query(k), dsresults[:n])
-    test_query(Query(k, limit=n), dsresults[:n])
-    test_query(Query(k, limit=n/2), dsresults[:n/2])
-    test_query(Query(k, offset=n/2), dsresults[n/2:])
-    test_query(Query(k, offset=n/3, limit=n/3), dsresults[n/3: 2*(n/3)])
+    test_query(Query(k), slice(0, n))
+    test_query(Query(k, limit=n), slice(0, n))
+    test_query(Query(k, limit=n/2), slice(0, n/2))
+    test_query(Query(k, offset=n/2), slice(n/2, n))
+    test_query(Query(k, offset=n/3, limit=n/3), slice(n/3, 2*(n/3)))
     del k
     del n
 
@@ -118,9 +119,94 @@ class TestDictionaryDatastore(TestDatastore):
     self.subtest_simple(stores)
 
 
+class TestKeyTransformDatastore(TestDatastore):
+
+  def test_simple(self):
+
+    s1 = datastore.KeyTransformDatastore(datastore.DictDatastore())
+    s2 = datastore.KeyTransformDatastore(datastore.DictDatastore())
+    s3 = datastore.KeyTransformDatastore(datastore.DictDatastore())
+    stores = [s1, s2, s3]
+
+    self.subtest_simple(stores)
+
+  def test_reverse_transform(self):
+
+    def transform(key):
+      return key.reverse
+
+    ds = datastore.DictDatastore()
+    kt = datastore.KeyTransformDatastore(ds, keytransform=transform)
+
+    k1 = Key('/a/b/c')
+    k2 = Key('/c/b/a')
+    self.assertFalse(ds.contains(k1))
+    self.assertFalse(ds.contains(k2))
+    self.assertFalse(kt.contains(k1))
+    self.assertFalse(kt.contains(k2))
+
+    ds.put(k1, 'abc')
+    self.assertEqual(ds.get(k1), 'abc')
+    self.assertFalse(ds.contains(k2))
+    self.assertFalse(kt.contains(k1))
+    self.assertEqual(kt.get(k2), 'abc')
+
+    kt.put(k1, 'abc')
+    self.assertEqual(ds.get(k1), 'abc')
+    self.assertEqual(ds.get(k2), 'abc')
+    self.assertEqual(kt.get(k1), 'abc')
+    self.assertEqual(kt.get(k2), 'abc')
+
+    ds.delete(k1)
+    self.assertFalse(ds.contains(k1))
+    self.assertEqual(ds.get(k2), 'abc')
+    self.assertEqual(kt.get(k1), 'abc')
+    self.assertFalse(kt.contains(k2))
+
+    kt.delete(k1)
+    self.assertFalse(ds.contains(k1))
+    self.assertFalse(ds.contains(k2))
+    self.assertFalse(kt.contains(k1))
+    self.assertFalse(kt.contains(k2))
+
+  def test_lowercase_transform(self):
+
+    def transform(key):
+      return Key(str(key).lower())
+
+    ds = datastore.DictDatastore()
+    lds = datastore.KeyTransformDatastore(ds, keytransform=transform)
+
+    k1 = Key('hello')
+    k2 = Key('HELLO')
+    k3 = Key('HeLlo')
+
+    ds.put(k1, 'world')
+    ds.put(k2, 'WORLD')
+
+    self.assertEqual(ds.get(k1), 'world')
+    self.assertEqual(ds.get(k2), 'WORLD')
+    self.assertFalse(ds.contains(k3))
+
+    self.assertEqual(lds.get(k1), 'world')
+    self.assertEqual(lds.get(k2), 'world')
+    self.assertEqual(lds.get(k3), 'world')
+
+    def test(key, val):
+      lds.put(key, val)
+      self.assertEqual(lds.get(k1), val)
+      self.assertEqual(lds.get(k2), val)
+      self.assertEqual(lds.get(k3), val)
+
+    test(k1, 'a')
+    test(k2, 'b')
+    test(k3, 'c')
+
+
+
 class TestLowercaseKeyDatastore(TestDatastore):
 
-  def test_dictionary(self):
+  def test_simple(self):
 
     s1 = datastore.LowercaseKeyDatastore(datastore.DictDatastore())
     s2 = datastore.LowercaseKeyDatastore(datastore.DictDatastore())
@@ -130,6 +216,79 @@ class TestLowercaseKeyDatastore(TestDatastore):
     self.subtest_simple(stores)
 
 
+  def test_lowercase(self):
+
+    ds = datastore.DictDatastore()
+    lds = datastore.LowercaseKeyDatastore(ds)
+
+    k1 = Key('hello')
+    k2 = Key('HELLO')
+    k3 = Key('HeLlo')
+
+    ds.put(k1, 'world')
+    ds.put(k2, 'WORLD')
+
+    self.assertEqual(ds.get(k1), 'world')
+    self.assertEqual(ds.get(k2), 'WORLD')
+    self.assertFalse(ds.contains(k3))
+
+    self.assertEqual(lds.get(k1), 'world')
+    self.assertEqual(lds.get(k2), 'world')
+    self.assertEqual(lds.get(k3), 'world')
+
+    def test(key, val):
+      lds.put(key, val)
+      self.assertEqual(lds.get(k1), val)
+      self.assertEqual(lds.get(k2), val)
+      self.assertEqual(lds.get(k3), val)
+
+    test(k1, 'a')
+    test(k2, 'b')
+    test(k3, 'c')
+
+
+class TestNamespaceDatastore(TestDatastore):
+
+  def test_simple(self):
+
+    s1 = datastore.NamespaceDatastore(Key('a'), datastore.DictDatastore())
+    s2 = datastore.NamespaceDatastore(Key('b'), datastore.DictDatastore())
+    s3 = datastore.NamespaceDatastore(Key('c'), datastore.DictDatastore())
+    stores = [s1, s2, s3]
+
+    self.subtest_simple(stores)
+
+
+  def test_namespace(self):
+
+    k1 = Key('/c/d')
+    k2 = Key('/a/b')
+    k3 = Key('/a/b/c/d')
+
+    ds = datastore.DictDatastore()
+    nd = datastore.NamespaceDatastore(k2, ds)
+
+    ds.put(k1, 'cd')
+    ds.put(k3, 'abcd')
+
+    self.assertEqual(ds.get(k1), 'cd')
+    self.assertFalse(ds.contains(k2))
+    self.assertEqual(ds.get(k3), 'abcd')
+
+    self.assertEqual(nd.get(k1), 'abcd')
+    self.assertFalse(nd.contains(k2))
+    self.assertFalse(nd.contains(k3))
+
+    def test(key, val):
+      nd.put(key, val)
+      self.assertEqual(nd.get(key), val)
+      self.assertFalse(ds.contains(key))
+      self.assertFalse(nd.contains(k2.child(key)))
+      self.assertEqual(ds.get(k2.child(key)), val)
+
+    for i in range(0, 10):
+      test(Key(str(i)), 'val%d' % i)
+
 
 class TestDatastoreCollection(TestDatastore):
 
